/***************************************************************************
                          extended_browser.cpp  -  description
                             -------------------
    begin                : Wed Jul 16 2003
    copyright            : (C) 2003 by George Goh
    email                : georgegohkokleong@hotmail.com
 ***************************************************************************/

#include "extended_browser.h"
#include <FL/Fl.H>
#include <FL/x.H>
#include <cstdio>
#include <iostream>
#include <string>

using namespace GUI;
using namespace std;

//#define EXTENDED_BROWSER_TEST
#ifdef EXTENDED_BROWSER_TEST
#define ENTER_KEY 0x0d
#define BACK_KEY  0x08
#define UP_KEY    0xf802
#define DOWN_KEY  0xf803
GR_WINDOW_ID  fl_window;
GR_GC_ID      fl_gc;
Extended_Browser *test;

void event_handler(GR_EVENT *event);

int main(void)
{
  if(GrOpen() < 0) {
    fprintf(stderr, "GrOpen failed\n");
    exit(1);
  }
  
  fl_gc=GrNewGC();
  GrSetGCBackground(fl_gc,LTGRAY);
  GrSetGCForeground(fl_gc,BLACK);
  
  fl_window=GrNewWindow(GR_ROOT_WINDOW_ID, 
                        0,0,640,480,0,
                        LTGRAY,WHITE);
                        
  GrSelectEvents(fl_window, GR_EVENT_MASK_EXPOSURE |
                            GR_EVENT_MASK_KEY_DOWN);
                            
  
  vector<string> list;
  vector<string> emptyList;
  list.push_back("Item 1");
  list.push_back("Item 2");
  list.push_back("Item 3");
  list.push_back("Item 4");
  list.push_back("Item 5");
  list.push_back("Item 6");
  list.push_back("Item 7");
  list.push_back("Item 8");  
  test=new Extended_Browser(200,50,320,240,list,emptyList,30);
  
  GrMapWindow(fl_window);
  
  GrMainLoop(event_handler);
}
  
void event_handler(GR_EVENT *event)
{
  switch(event->type) {
    case GR_EVENT_TYPE_EXPOSURE:
      test->draw(); 
      break;
      
    case GR_EVENT_TYPE_KEY_DOWN:
      printf("Key pressed\n");
      test->handle(event);
      test->draw();
      GrSetGCRegion(fl_gc,0);
      GrText(fl_window,fl_gc,10,30,(GR_CHAR*)"Check this out!",-1,GR_TFASCII);
      break;
  }
}
  
#endif //EXTENDED_BROWSER_TEST
// Set up the list of items, and the associated tips, using _fontSize
Extended_Browser::Extended_Browser(int _x,int _y,int _w,int _h,
                                   vector<string> &inList,
                                   vector<string> &listTips, int _fontSize) {

  x=_x;
  y=_y;
  w=_w;
  h=_h;

  #ifdef DEBUG
    printf("Extended_Browser: x=%d y=%d w=%d h=%d\n", x,y,w,h);
  #endif
  clipRegion = GrNewRegion();
  GR_RECT clipRect;
  clipRect.x=x; clipRect.y=y; clipRect.width=w; clipRect.height=h-1;
  GrUnionRectWithRegion(clipRegion,&clipRect);
  
  fontId = GrCreateFont((GR_CHAR*)"arial", _fontSize, NULL);
  line_h = _fontSize + 4;
  setOptionList(inList, listTips);
}

Extended_Browser::~Extended_Browser(){
}

// set up option list and tip list and initialise the highlight the first item
void Extended_Browser::setOptionList(vector<string> &inList, vector<string> &listTips)
{

  items.clear();
  listTips.clear();
  
  items=inList;
  size = items.size();
  optionTips=listTips;

  position=1;

  resolveItemDimensions();
  drawnOnce = false;
}

// Find out the dimensions of each item
void Extended_Browser::resolveItemDimensions()
{
  items_onscreen.clear();
  
  // allow for the border
  int item_x=x+1;
  int item_w=w-2;
  formText temp;

  items_onscreen.push_back(temp);
  
  for(unsigned int i=1; i<=size; ++i) {
    temp = items[i-1];
    temp.x = item_x;
    temp.y = (i-1)*line_h+y+1;
    temp.w = item_w;
    temp.h = line_h;
    temp.damage = true;
    items_onscreen.push_back(temp);
  }
}

void Extended_Browser::draw()
{
  if(drawnOnce == false) {
    GrSetGCForeground(fl_gc,WHITE);
    GrFillRect(fl_window,fl_gc,x,y,w,h);
    GrSetGCForeground(fl_gc,BLACK);
    GrRect(fl_window,fl_gc,x,y,w,h);
    drawnOnce = true;
  }

  GrSetGCRegion(fl_gc,clipRegion);
  GrSetGCUseBackground(fl_gc, 0);
  GrSetGCFont(fl_gc,fontId);
  
  // only draw an item if it is damaged
  for(unsigned int i=1; i<=size; ++i) {
    if(items_onscreen[i].damage == true) {
      
      // highlight if current item drawn is selected
      if((unsigned int)position==i) GrSetGCForeground(fl_gc,LTGRAY);
      else GrSetGCForeground(fl_gc,WHITE);
      
      GrFillRect(fl_window,fl_gc,items_onscreen[i].x,
                                 items_onscreen[i].y,
                                 items_onscreen[i].w,
                                 items_onscreen[i].h);

      GrSetGCForeground(fl_gc,BLACK);
      GrText(fl_window,fl_gc,5+items_onscreen[i].x,
                             items_onscreen[i].y+line_h-5,
                             (char*)items_onscreen[i].c_str(),-1,GR_TFASCII);
      
      items_onscreen[i].damage=false;
    }
  }
}

void Extended_Browser::value(int _inVal)
{
  items_onscreen[position].damage=true;
  position = _inVal;
  items_onscreen[position].damage=true;
}

int Extended_Browser::value()
{
  return position;
}


int Extended_Browser::handle(GR_EVENT* event) {
  
  if (event->type == GR_EVENT_TYPE_KEY_DOWN) {
    switch(event->keystroke.ch) {
      case ENTER_KEY:
      case BACK_KEY:
        printf("Enter or back key pressed.\n");
        // HANDLING FUNCTIONS HERE
        return 1;
      
      case UP_KEY:
        if(value()<=1) return 1;
        else value(value()-1);
        // HANDLING FUNCTIONS HERE
        return 1;
      
      case DOWN_KEY:
        if(value()>=size) return 1;
        else value(value()+1);
        // HANDLING FUNCTIONS HERE
        return 1;
    } 
  }
  return 1;
}

